@@ -5,7 +5,7 @@ use warnings;
 
 use utf8;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 binmode STDERR, ':encoding(UTF-8)';
 binmode STDOUT, ':encoding(UTF-8)';
@@ -25,11 +25,11 @@ my %params = (
 );
 
 sub usage { return
-	"Usage : $0 [--I 'a-json-string' | --i 'afile.json'] [--o afile] [--escape-unicode|-e] [--pretty] [--terse] [--no-indent]\n"
+	"Usage : $0 [--I 'a-json-string' | --i 'afile.json'] [--o afile] [--(no-)escape-unicode|-e] [--(no-)pretty] [--(no-)terse] [--(no-)indent]\n"
 	."\nIt will read a JSON string from command line (-I), or from a file (-i)\n"
-	."\nor from STDIN.\n"
+	."\nor from STDIN (beware 4K limit on linux terminal, see CAVEATS for workaround).\n"
 	."It will print its contents as a Perl variable (dump) to STDOUT or to a file (--o).\n"
-	."It can escape/un-escape unicode characters (--escape-unicode) and/or --terse and --no-indent.\n"
+	."It can escape/un-escape unicode characters (--escape-unicode) and/or --terse and/or --indent.\n"
 }
 if( ! Getopt::Long::GetOptions(
   'i=s' => \$INPUT_FILE,
@@ -71,11 +71,26 @@ json2perl.pl : convert JSON data to a Perl variable (dump) which can be parsed o
 
 =head1 VERSION
 
-Version 0.06
+Version 0.07
 
 =head1 SYNOPSIS
 
-    json2perl.pl -i "input.json" -o "output.perl" --no-escape-unicode --terse --no-indent
+    json2perl.pl -i "input.json" -o "output.pl" --no-escape-unicode --terse --no-indent
+
+    json2perl.pl -e < "input.json" > "output.pl"
+
+    # press CTRL-D when done typing JSON to STDIN
+    # input must be less than 4K long!
+    json2perl.pl
+
+    # Read input from clipboard or write output to clipboard
+    # Only in: Unix / Linux / OSX
+    # (must have already installed xclip or xsel or pbpaste (on OSX))
+    json2json.pl -e < $(xclip -o)
+    json2json.pl -e < $(pbaste)
+    # write the output to the clipboard for further pasting
+    json2json.pl -i input.json | xclip -i
+    # clicking mouse's middle-button will paste the result
 
 =head1 USAGE
 
@@ -85,22 +100,26 @@ Options:
 
 =over 4
 
-=item C<--i filename> : specify a filename which contains a JSON
+=item * C<--i filename> : specify a filename which contains a JSON
 data structure.
 
-=item C<--I "string"> : specify a string  which contains a JSON
+=item * C<--I "string"> : specify a string  which contains a JSON
 data structure.
 
-=item C<--o outputfilename> : specify the output filename to write
+=item * C<--o outputfilename> : specify the output filename to write
 the result to, which will be as a Perl variable, as a dump,
 which can be parsed or eval'ed from any Perl script.
 
-=item C<--escape-unicode> : it will escape all unicode characters, and
-convert them to something like "\u0386"
+=item * C<--escape-unicode> : it will escape all unicode characters, and
+convert them to something like "\u0386". This is the default option.
 
-=item C<--terse> : Terse form of output (no $VAR1)
+=item * C<--no-escape-unicode> : it will NOT escape unicode characters. Output
+will not contain "\u0386" or "\x{386}" but "α" (that's a greek alpha).
 
-=item C<--no-indent> : do not use indentation
+=item * C<--terse> / C<--no-terse> : Terse form of output (no $VAR1 for example).
+The second is the default option.
+
+=item * C<--indent> / C<--no-indent> : do not use indentation. The first is the default option.
 
 =back
 
@@ -110,6 +129,17 @@ a file redirection C<< json2perl.pl < inputfile.json > outputfile.perl >>
 
 For more information see L<Data::Roundtrip>.
 
+=head1 CAVEATS
+
+Under Unix/Linux,
+the maximum number of characters that can be read
+on a terminal is 4096. So, in reading-from-STDIN mode
+beware how much you type or how much you copy-paste
+onto the script. If it complains about malformed input
+then this is the case. The workaround is to type/paste
+onto a file and operate on that using C<< --i afile >>
+or redirection C<< < afile >>.
+
 =head1 AUTHOR
 
 Andreas Hadjiprocopis, C<< <bliako at cpan.org> / <andreashad2 at gmail.com> >>
